@@ -22,6 +22,7 @@
 mod constants;
 mod diff;
 mod display;
+mod exit_codes;
 mod files;
 mod line_parser;
 mod lines;
@@ -40,6 +41,7 @@ use crate::parse::syntax;
 use diff::changes::ChangeMap;
 use diff::dijkstra::ExceededGraphLimit;
 use display::context::opposite_positions;
+use exit_codes::{EXIT_FOUND_CHANGES, EXIT_SUCCESS};
 use files::{
     guess_content, read_files_or_die, read_or_die, relative_paths_in_either, ProbableFileKind,
 };
@@ -58,6 +60,8 @@ use diff::sliders::fix_all_sliders;
 use options::{DisplayMode, DisplayOptions, FileArgument, Mode, DEFAULT_TAB_WIDTH};
 use owo_colors::OwoColorize;
 use rayon::prelude::*;
+use std::sync::atomic::{AtomicBool, Ordering};
+use std::sync::Arc;
 use std::{env, path::Path};
 use summary::{DiffResult, FileContent};
 use syntax::init_next_prev;
@@ -157,6 +161,8 @@ fn main() {
             byte_limit,
             display_options,
             missing_as_empty,
+            set_exit_code,
+            check_only,
             language_override,
             lhs_path,
             rhs_path,
@@ -175,22 +181,46 @@ fn main() {
                 );
             }
 
+            let encountered_changes = Arc::new(AtomicBool::new(false));
             match (&lhs_path, &rhs_path) {
                 (
                     options::FileArgument::NamedPath(lhs_path),
                     options::FileArgument::NamedPath(rhs_path),
                 ) if lhs_path.is_dir() && rhs_path.is_dir() => {
+                    // We want to diff files in the directory in
+                    // parallel, but print the results serially (to
+                    // prevent display interleaving).
+                    // https://github.com/rayon-rs/rayon/issues/210#issuecomment-551319338
+                    let (send, recv) = std::sync::mpsc::sync_channel(1);
+
+                    let encountered_changes = encountered_changes.clone();
+                    let print_options = display_options.clone();
+
+                    let printing_thread = std::thread::spawn(move || {
+                        for diff_result in recv.into_iter() {
+                            print_diff_result(&print_options, &diff_result);
+
+                            if diff_result.has_reportable_change() {
+                                encountered_changes.store(true, Ordering::Relaxed);
+                            }
+                        }
+                    });
+
                     diff_directories(
                         lhs_path,
                         rhs_path,
                         &display_options,
                         graph_limit,
                         byte_limit,
+                        check_only,
                         language_override,
                     )
-                    .for_each(|diff_result| {
-                        print_diff_result(&display_options, &diff_result);
-                    });
+                    .try_for_each_with(send, |s, diff_result| s.send(diff_result))
+                    .expect("Receiver should be connected");
+
+                    printing_thread
+                        .join()
+                        .expect("Printing thread should not panic");
                 }
                 _ => {
                     let diff_result = diff_file(
@@ -202,11 +232,23 @@ fn main() {
                         missing_as_empty,
                         graph_limit,
                         byte_limit,
+                        check_only,
                         language_override,
                     );
                     print_diff_result(&display_options, &diff_result);
+
+                    if diff_result.has_reportable_change() {
+                        encountered_changes.store(true, Ordering::Relaxed);
+                    }
                 }
             }
+
+            let exit_code = if set_exit_code && encountered_changes.load(Ordering::Relaxed) {
+                EXIT_FOUND_CHANGES
+            } else {
+                EXIT_SUCCESS
+            };
+            std::process::exit(exit_code);
         }
     };
 }
@@ -231,6 +273,7 @@ fn diff_file(
     missing_as_empty: bool,
     graph_limit: usize,
     byte_limit: usize,
+    check_only: bool,
     language_override: Option<parse::guess_language::Language>,
 ) -> DiffResult {
     let (lhs_bytes, rhs_bytes) = read_files_or_die(lhs_path, rhs_path, missing_as_empty);
@@ -241,9 +284,10 @@ fn diff_file(
         rhs_path,
         &lhs_bytes,
         &rhs_bytes,
-        display_options.tab_width,
+        display_options,
         graph_limit,
         byte_limit,
+        check_only,
         language_override,
     )
 }
@@ -255,9 +299,10 @@ fn diff_file_content(
     rhs_path: &FileArgument,
     lhs_bytes: &[u8],
     rhs_bytes: &[u8],
-    tab_width: usize,
+    display_options: &DisplayOptions,
     graph_limit: usize,
     byte_limit: usize,
+    check_only: bool,
     language_override: Option<parse::guess_language::Language>,
 ) -> DiffResult {
     let (mut lhs_src, mut rhs_src) = match (guess_content(lhs_bytes), guess_content(rhs_bytes)) {
@@ -267,18 +312,21 @@ fn diff_file_content(
                 rhs_display_path: rhs_display_path.into(),
                 language: None,
                 detected_language: None,
-                lhs_src: FileContent::Binary(lhs_bytes.to_vec()),
-                rhs_src: FileContent::Binary(rhs_bytes.to_vec()),
+                lhs_src: FileContent::Binary,
+                rhs_src: FileContent::Binary,
                 lhs_positions: vec![],
                 rhs_positions: vec![],
+                hunks: vec![],
+                has_byte_changes: lhs_bytes != rhs_bytes,
+                has_syntactic_changes: false,
             };
         }
         (ProbableFileKind::Text(lhs_src), ProbableFileKind::Text(rhs_src)) => (lhs_src, rhs_src),
     };
 
     // TODO: don't replace tab characters inside string literals.
-    lhs_src = replace_tabs(&lhs_src, tab_width);
-    rhs_src = replace_tabs(&rhs_src, tab_width);
+    lhs_src = replace_tabs(&lhs_src, display_options.tab_width);
+    rhs_src = replace_tabs(&rhs_src, display_options.tab_width);
 
     // Ignore the trailing newline, if present.
     // TODO: highlight if this has changes (#144).
@@ -311,6 +359,9 @@ fn diff_file_content(
             rhs_src: FileContent::Text("".into()),
             lhs_positions: vec![],
             rhs_positions: vec![],
+            hunks: vec![],
+            has_byte_changes: false,
+            has_syntactic_changes: false,
         };
     }
 
@@ -331,6 +382,25 @@ fn diff_file_content(
 
             init_all_info(&lhs, &rhs);
 
+            if check_only {
+                let lang_name = language.map(|l| language_name(l).into());
+                let has_syntactic_changes = lhs != rhs;
+
+                return DiffResult {
+                    lhs_display_path: lhs_display_path.into(),
+                    rhs_display_path: rhs_display_path.into(),
+                    language: lang_name,
+                    detected_language: language,
+                    lhs_src: FileContent::Text(lhs_src),
+                    rhs_src: FileContent::Text(rhs_src),
+                    lhs_positions: vec![],
+                    rhs_positions: vec![],
+                    hunks: vec![],
+                    has_byte_changes: true,
+                    has_syntactic_changes,
+                };
+            }
+
             let mut change_map = ChangeMap::default();
             let possibly_changed = if env::var("DFT_DBG_KEEP_UNCHANGED").is_ok() {
                 vec![(lhs.clone(), rhs.clone())]
@@ -389,6 +459,20 @@ fn diff_file_content(
         }
     };
 
+    let opposite_to_lhs = opposite_positions(&lhs_positions);
+    let opposite_to_rhs = opposite_positions(&rhs_positions);
+
+    let hunks = matched_pos_to_hunks(&lhs_positions, &rhs_positions);
+    let hunks = merge_adjacent(
+        &hunks,
+        &opposite_to_lhs,
+        &opposite_to_rhs,
+        lhs_src.max_line(),
+        rhs_src.max_line(),
+        display_options.num_context_lines as usize,
+    );
+    let has_syntactic_changes = !hunks.is_empty();
+
     DiffResult {
         lhs_display_path: lhs_display_path.into(),
         rhs_display_path: rhs_display_path.into(),
@@ -398,6 +482,9 @@ fn diff_file_content(
         rhs_src: FileContent::Text(rhs_src),
         lhs_positions,
         rhs_positions,
+        hunks,
+        has_byte_changes: true,
+        has_syntactic_changes,
     }
 }
 
@@ -413,6 +500,7 @@ fn diff_directories<'a>(
     display_options: &DisplayOptions,
     graph_limit: usize,
     byte_limit: usize,
+    check_only: bool,
     language_override: Option<parse::guess_language::Language>,
 ) -> impl ParallelIterator<Item = DiffResult> + 'a {
     let display_options = display_options.clone();
@@ -437,6 +525,7 @@ fn diff_directories<'a>(
             true,
             graph_limit,
             byte_limit,
+            check_only,
             language_override,
         )
     })
@@ -445,21 +534,10 @@ fn diff_directories<'a>(
 fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
     match (&summary.lhs_src, &summary.rhs_src) {
         (FileContent::Text(lhs_src), FileContent::Text(rhs_src)) => {
-            let opposite_to_lhs = opposite_positions(&summary.lhs_positions);
-            let opposite_to_rhs = opposite_positions(&summary.rhs_positions);
-
-            let hunks = matched_pos_to_hunks(&summary.lhs_positions, &summary.rhs_positions);
-            let hunks = merge_adjacent(
-                &hunks,
-                &opposite_to_lhs,
-                &opposite_to_rhs,
-                lhs_src.max_line(),
-                rhs_src.max_line(),
-                display_options.num_context_lines as usize,
-            );
+            let hunks = &summary.hunks;
 
             let lang_name = summary.language.clone().unwrap_or_else(|| "Text".into());
-            if hunks.is_empty() {
+            if !summary.has_syntactic_changes {
                 if display_options.print_unchanged {
                     println!(
                         "{}",
@@ -483,6 +561,29 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                 return;
             }
 
+            if summary.has_syntactic_changes && hunks.is_empty() {
+                println!(
+                    "{}",
+                    display::style::header(
+                        &summary.lhs_display_path,
+                        &summary.rhs_display_path,
+                        1,
+                        1,
+                        &lang_name,
+                        display_options
+                    )
+                );
+                if lang_name == "Text" {
+                    // TODO: there are other Text names now, so
+                    // they will hit the second case incorrectly.
+                    println!("Has changes.\n");
+                } else {
+                    println!("Has syntactic changes.\n");
+                }
+
+                return;
+            }
+
             match display_options.display_mode {
                 DisplayMode::Inline => {
                     display::inline::print(
@@ -491,7 +592,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         display_options,
                         &summary.lhs_positions,
                         &summary.rhs_positions,
-                        &hunks,
+                        hunks,
                         &summary.lhs_display_path,
                         &summary.rhs_display_path,
                         &lang_name,
@@ -500,7 +601,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                 }
                 DisplayMode::SideBySide | DisplayMode::SideBySideShowBoth => {
                     display::side_by_side::print(
-                        &hunks,
+                        hunks,
                         display_options,
                         &summary.lhs_display_path,
                         &summary.rhs_display_path,
@@ -514,9 +615,8 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                 }
             }
         }
-        (FileContent::Binary(lhs_bytes), FileContent::Binary(rhs_bytes)) => {
-            let changed = lhs_bytes != rhs_bytes;
-            if display_options.print_unchanged || changed {
+        (FileContent::Binary, FileContent::Binary) => {
+            if display_options.print_unchanged || summary.has_byte_changes {
                 println!(
                     "{}",
                     display::style::header(
@@ -528,14 +628,15 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         display_options
                     )
                 );
-                if changed {
+                if summary.has_byte_changes {
                     println!("Binary contents changed.");
                 } else {
                     println!("No changes.");
                 }
             }
         }
-        (_, FileContent::Binary(_)) | (FileContent::Binary(_), _) => {
+        (FileContent::Text(_), FileContent::Binary)
+        | (FileContent::Binary, FileContent::Text(_)) => {
             // We're diffing a binary file against a text file.
             println!(
                 "{}",
@@ -558,7 +659,7 @@ mod tests {
     use std::ffi::OsStr;
 
     use super::*;
-    use crate::options::{DEFAULT_BYTE_LIMIT, DEFAULT_GRAPH_LIMIT, DEFAULT_TAB_WIDTH};
+    use crate::options::{DEFAULT_BYTE_LIMIT, DEFAULT_GRAPH_LIMIT};
 
     #[test]
     fn test_diff_identical_content() {
@@ -570,9 +671,10 @@ mod tests {
             &FileArgument::from_path_argument(OsStr::new("foo.el")),
             s.as_bytes(),
             s.as_bytes(),
-            DEFAULT_TAB_WIDTH,
+            &DisplayOptions::default(),
             DEFAULT_GRAPH_LIMIT,
             DEFAULT_BYTE_LIMIT,
+            false,
             None,
         );
 
