@@ -1,12 +1,14 @@
 //! CLI option parsing.
 
-use std::{borrow::Borrow, env, ffi::OsStr, path::PathBuf};
+use std::{borrow::Borrow, env, ffi::OsStr, path::Path, path::PathBuf};
 
 use atty::Stream;
 use clap::{crate_authors, crate_description, crate_version, Arg, Command};
 use const_format::formatcp;
 
-use crate::{display::style::BackgroundColor, parse::guess_language};
+use crate::{
+    display::style::BackgroundColor, exit_codes::EXIT_BAD_ARGUMENTS, parse::guess_language,
+};
 
 pub const DEFAULT_BYTE_LIMIT: usize = 1_000_000;
 // Chosen experimentally: this is sufficiently many for all the sample
@@ -37,6 +39,22 @@ pub struct DisplayOptions {
     pub syntax_highlight: bool,
 }
 
+impl Default for DisplayOptions {
+    fn default() -> Self {
+        Self {
+            background_color: BackgroundColor::Dark,
+            use_color: false,
+            display_mode: DisplayMode::SideBySide,
+            print_unchanged: true,
+            tab_width: 8,
+            display_width: 80,
+            num_context_lines: 3,
+            in_vcs: false,
+            syntax_highlight: true,
+        }
+    }
+}
+
 fn app() -> clap::Command<'static> {
     Command::new("Difftastic")
         .override_usage(USAGE)
@@ -134,6 +152,16 @@ fn app() -> clap::Command<'static> {
                 .default_value("on")
                 .help("Enable or disable syntax highlighting.")
         )
+        .arg(
+            Arg::new("exit-code").long("exit-code")
+                .env("DFT_EXIT_CODE")
+                .help("Set the exit code to 1 if there are syntactic changes in any text files, or byte changes in any binary files.")
+        )
+        .arg(
+            Arg::new("check-only").long("check-only")
+                .env("DFT_CHECK_ONLY")
+                .help("Report whether there are any syntactic changes, but don't calculate them. Much faster.")
+        )
         .arg(
             Arg::new("skip-unchanged").long("skip-unchanged")
                 .help("Don't display anything if a file is unchanged.")
@@ -197,6 +225,16 @@ pub enum FileArgument {
     DevNull,
 }
 
+fn relative_to_current(path: &Path) -> PathBuf {
+    if let Ok(current_path) = std::env::current_dir() {
+        if let Ok(rel_path) = path.strip_prefix(current_path) {
+            return rel_path.into();
+        }
+    }
+
+    path.into()
+}
+
 impl FileArgument {
     /// Return a `FileArgument` representing this command line
     /// argument.
@@ -218,7 +256,7 @@ impl FileArgument {
 
     pub fn display(&self) -> String {
         match self {
-            FileArgument::NamedPath(path) => path.display().to_string(),
+            FileArgument::NamedPath(path) => relative_to_current(path).display().to_string(),
             FileArgument::Stdin => "(stdin)".to_string(),
             FileArgument::DevNull => "/dev/null".to_string(),
         }
@@ -231,6 +269,8 @@ pub enum Mode {
         byte_limit: usize,
         display_options: DisplayOptions,
         missing_as_empty: bool,
+        set_exit_code: bool,
+        check_only: bool,
         language_override: Option<guess_language::Language>,
         /// The path where we can read the LHS file. This is often a
         /// temporary file generated by source control.
@@ -357,7 +397,7 @@ pub fn parse_args() -> Mode {
             }
             eprintln!("USAGE:\n\n    {}\n", USAGE);
             eprintln!("For more information try --help");
-            std::process::exit(1);
+            std::process::exit(EXIT_BAD_ARGUMENTS);
         }
     };
 
@@ -423,6 +463,10 @@ pub fn parse_args() -> Mode {
     // file on all platforms?
     let missing_as_empty = matches.is_present("missing-as-empty");
 
+    let set_exit_code = matches.is_present("exit-code");
+
+    let check_only = matches.is_present("check-only");
+
     let display_options = DisplayOptions {
         background_color,
         use_color,
@@ -440,6 +484,8 @@ pub fn parse_args() -> Mode {
         byte_limit,
         display_options,
         missing_as_empty,
+        set_exit_code,
+        check_only,
         language_override,
         lhs_path,
         rhs_path,
